"""

abstract:
---------
         something between global params, constants,
         configuration variables, ini file...

contents:
---------
      + low-level limits
         - query size
         - max upload size
         - doc parsing batch size
         - word extraction batch size

      + main process config
         - wordlist generation params
         - graph creation params
         - £TODO sequence of transformations "custom pipeline"

      + subprocess config
         - tagger services and functions
         - stemmer services

"""
import os
import re
import importlib
from .settings import BASE_DIR

# XXX Originally defined here, imported here for backward-compatibility,
#     should be removed later.
from .models.nodes_constants import NODETYPES, LISTTYPES, INDEXED_HYPERDATA, \
                                    RESOURCETYPES, get_resource, get_resource_by_name, \
                                    load_parser, load_crawler

# user parameters----------------------------------------
USER_LANG = ["fr", "en"]

# taggers -----------------------------------------------
def get_tagger(lang):
    '''
    lang => observed language[0] => Tagger
    '''
    name = LANGUAGES[lang]["tagger"]
    module = "gargantext.util.taggers.%s" %(name)
    module = importlib.import_module(module, "")
    tagger = getattr(module, name)
    return tagger()


# Supported languages and taggers ---------------------------------------------
#first declare the tagger using a string
#and it will be imported into gargantext.utils.taggers
LANGUAGES = {
    'en': {
        #'tagger': 'EnglishMeltTagger',
        #'tagger': "TurboTagger",
        'tagger': 'NltkTagger',
    },
    'fr': {
        #'tagger': "FrenchMeltTagger",
        #'tagger': 'TreeTagger',
        'tagger': 'NltkTagger',
    },
}

def load_tagger(lang):
    '''
    given a LANG load the corresponding tagger
    lang(str) > Tagger(Object)
    '''
    filename = LANGUAGES[lang]["tagger"]
    module = 'gargantext.util.taggers.%s' %(filename)
    module = importlib.import_module(module)
    return getattr(module, filename)()


# linguistic extraction parameters ---------------------------------------------

DEFAULT_RANK_CUTOFF_RATIO      = .75         # MAINLIST maximum terms in %

DEFAULT_RANK_HARD_LIMIT        = 5000        # MAINLIST maximum terms abs
                                             # (makes COOCS larger ~ O(N²) /!\)

DEFAULT_COOC_THRESHOLD          = 3          # inclusive minimum for COOCS coefs
                                             # (makes COOCS more sparse)

DEFAULT_MAPLIST_MAX             = 350        # MAPLIST maximum terms

DEFAULT_MAPLIST_MONOGRAMS_RATIO = .2         # quota of monograms in MAPLIST
                                             # (vs multigrams = 1-mono)

DEFAULT_MAPLIST_GENCLUSION_RATIO = .6        # quota of top genclusion in MAPLIST
                                             # (vs top specclusion = 1-gen)

DEFAULT_MAX_NGRAM_LEN           = 7          # limit used after POStagging rule
                                             # (initial ngrams number is a power law of this /!\)
                                             # (and most longer ngrams have tiny freq anyway)

DEFAULT_ALL_LOWERCASE_FLAG      = True       # lowercase ngrams before recording
                                             # them to their DB table
                                             # (potentially bad for acronyms but
                                             #  good for variants like same term
                                             #  occurring at sentence beginning)

DEFAULT_INDEX_SUBGRAMS         = False        # False <=> traditional
                                             # True  <=>
                                             #  when ngram is like:
                                             #  "very cool example"
                                             #  then also count: "very", "cool"
                                             #  "example", "very cool" and
                                             #  "cool example".
                                             #   (all 1 to n-1 length ngrams,
                                             #    at indexing after extraction)

# TAGGING options   -----------------------------------------
#activate lang detection?
DETECT_LANG = False
# Defaults INDEXED Fields for ngrams extraction
# put longest field first in order to make detection language more efficient
DEFAULT_INDEX_FIELDS            = ('abstract','title' )
# Grammar rules for chunking
RULE_JJNN   = "{<JJ.*>*<NN.*|>+<JJ.*>*}"
RULE_JJDTNN = "{<JJ.*>*<NN.*>+((<P|IN> <DT>? <JJ.*>* <NN.*>+ <JJ.*>*)|(<JJ.*>))*}"
RULE_TINA   = "^((VBD,|VBG,|VBN,|CD.?,|JJ.?,|\?,){0,2}?(N.?.?,|\?,)+?(CD.,)??)\
               +?((PREP.?|DET.?,|IN.?,|CC.?,|\?,)((VBD,|VBG,|VBN,|CD.?,|JJ.?,|\?\
               ,){0,2}?(N.?.?,|\?,)+?)+?)*?$"


# ngram lists import/export parameters -----------------------------------------
DEFAULT_CSV_DELIM              = '\t'        # for import/export CSV defaults
DEFAULT_CSV_DELIM_GROUP        = '|&|'



# Files ----------------------------------------------------------------
# uploads/.gitignore prevents corpora indexing
# copora can be either a folder or symlink towards specific partition
UPLOAD_DIRECTORY   = os.path.join(BASE_DIR, 'uploads/corpora')
UPLOAD_LIMIT       = 1024 * 1024 * 1024
DOWNLOAD_DIRECTORY = UPLOAD_DIRECTORY

# Processing -----------------------------------------------------------
# about batch processing...
BATCH_PARSING_SIZE          = 256    # how many new docs before db write
BATCH_NGRAMSEXTRACTION_SIZE = 3000   # how many new node-ngram relations before INTEGRATE


# Scrapers config
QUERY_SIZE_N_MAX     = 1000
QUERY_SIZE_N_DEFAULT = 1000

# Refresh corpora workflow status for project view's progressbar
PROJECT_VIEW_REFRESH_INTERVAL  = 3000     # 1st refresh in ms (then increasing arithmetically)
PROJECT_VIEW_MAX_REFRESH_ATTEMPTS = 10    # how many times before we give up

# ------------------------------------------------------------------------------
# Graph <=> nodes API parameters
# number of relevant publications shown
DEFAULT_N_DOCS_HAVING_NGRAM = 5

# ------------------------------------------------------------------------------
# Graph constraints to compute the graph:
# Modes: live graph generation, graph asynchronously computed or errors detected
# here are the maximum size of corpus and maplist required to compute the graph
graph_constraints = {'corpusMax' : 0
                    ,'corpusMin' : 0
                    ,'mapList'   : 50
                    }
