"""
# WARNING: to ensure consistency and retrocompatibility, lists should keep the
#   initial order (ie., new elements should be appended at the end of the lists)

abstract:
---------
        constants and utility functions related to nodes
        initially in gargantext.constants

contents:
---------
      + database constants/ontology
         - nodetypes
            (db int <=> named types <=> python code)

      + main process config
         - resourcetypes config (~ input ontology)

      + subprocess config
         - crawling, import
         - parser services

"""

import importlib

from ..util.lists import WeightedList, UnweightedList, WeightedIndex, \
                         WeightedMatrix, Translations
from ..util.dates import datetime, convert_to_datetime

# types & models (nodes, lists, hyperdata, resource) ---------------------------------------------

NODETYPES = [
    # TODO separate id not array index, read by models.node
    None,                    # 0
    # documents hierarchy
    'USER',                  # 1
    'PROJECT',               # 2
    #RESOURCE should be here but last
    'CORPUS',                # 3
    'DOCUMENT',              # 4
    # lists
    'STOPLIST',              # 5
    'GROUPLIST',             # 6
    'MAINLIST',              # 7
    'MAPLIST',               # 8
    'COOCCURRENCES',         # 9
    # scores
    'OCCURRENCES',           # 10
    'SPECCLUSION',           # 11
    'CVALUE',                # 12
    'TFIDF-CORPUS',          # 13
    'TFIDF-GLOBAL',          # 14
    # docs subset
    'FAVORITES',             # 15
    # more scores (sorry!)

    'TIRANK-LOCAL',          # 16
    'TIRANK-GLOBAL',         # 17

    'GENCLUSION',            # 18
    'RESOURCE',              # 19
]

LISTTYPES = {
    'DOCUMENT'     : WeightedList,
    'GROUPLIST'    : Translations,   # todo remove "LIST" from name
    'STOPLIST'     : UnweightedList,
    'MAINLIST'     : UnweightedList,
    'MAPLIST'      : UnweightedList,
    'SPECCLUSION'  : WeightedList,
    'GENCLUSION'   : WeightedList,
    'OCCURRENCES'  : WeightedIndex,   # could be WeightedList
    'COOCCURRENCES': WeightedMatrix,
    'TFIDF-CORPUS' : WeightedIndex,
    'TFIDF-GLOBAL' : WeightedIndex,
    'TIRANK-LOCAL' : WeightedIndex,   # could be WeightedList
}
# 'OWNLIST'      : UnweightedList,    # £TODO use this for any term-level tags

INDEXED_HYPERDATA = {
    # TODO use properties during toolchain.hyperdata_indexing
    # (type, convert_to_db, convert_from_db)

    'count':
        { 'id'             : 1
        , 'type'           : int
        , 'convert_to_db'  : int
        , 'convert_from_db': int
        },

    'publication_date':
        { 'id'             : 2
        , 'type'           : datetime
        , 'convert_to_db'  : convert_to_datetime
        , 'convert_from_db': convert_to_datetime
        },

    'title':
        { 'id'             : 3
        , 'type'           : str
        , 'convert_to_db'  : str
        , 'convert_from_db': str
        },

    'authors':
        { 'id'             : 4
        , 'type'           : str
        , 'convert_to_db'  : str
        , 'convert_from_db': str
        },

    'journal':
        { 'id'             : 5
        , 'type'           : str
        , 'convert_to_db'  : str
        , 'convert_from_db': str
        },

    'abstract':
        { 'id'             : 6
        , 'type'           : str
        , 'convert_to_db'  : str
        , 'convert_from_db': str
        },

    # 'text':
    #     { 'id'             : 7
    #     , 'type'           : str
    #     , 'convert_to_db'  : str
    #     , 'convert_from_db': str
    #     },
    #
    # 'page':
    #     { 'id'             : 8
    #     , 'type'           : int
    #     , 'convert_to_db'  : int
    #     , 'convert_from_db': int
    #     },

}

# resources ---------------------------------------------
def get_resource(sourcetype):
    '''resource :: type => resource dict'''
    return RESOURCETYPES[sourcetype-1]

def get_resource_by_name(sourcename):
    '''resource :: name => resource dict'''
    for n in RESOURCETYPES:
        if str(n["name"]) == str(sourcename):
            return n

RESOURCETYPES = [
    {   "type": 1,
        'name': 'Europresse',
        'format': 'Europresse',
        'parser': "EuropresseParser",
        'file_formats':["zip", "txt"],
        'crawler': None,
    },
    {   'type': 2,
        'name': 'Jstor [RIS]',
        'format': 'RIS',
        'parser': "RISParser",
        'file_formats':["zip", "txt"],
        'crawler': None,
    },
    {   'type': 3,
        'name': 'Pubmed [XML]',
        'format': 'Pubmed',
        'parser': "PubmedParser",
        'file_formats':["zip", "xml"],
        'crawler': "PubmedCrawler",
    },
    {   'type': 4,
        'name': 'Scopus [RIS]',
        'format': 'RIS',
        'parser': "RISParser",
        'file_formats':["zip", "txt"],
        'crawler': None,
    },
    {   'type': 5,
        'name': 'Web of Science [ISI]',
        'format': 'ISI',
        'parser': "ISIParser",
        'file_formats':["zip", "txt", "isi"],
        #'crawler': "ISICrawler",
        'crawler': None,
    },
    {   'type': 6,
        'name': 'Zotero [RIS]',
        'format': 'RIS',
        'parser': 'RISParser',
        'file_formats':["zip", "ris", "txt"],
        'crawler': None,
    },
    {   'type': 7,
        'name': 'CSV',
        'format': 'CSV',
        'parser': 'CSVParser',
        'file_formats':["zip", "csv"],
        'crawler': None,
    },
    {   'type': 8,
        'name': 'ISTex',
        'format': 'json',
        'parser': "ISTexParser",
        'file_formats':["zip", "txt"],
        'crawler': None,
    },
   {    "type": 9,
        "name": 'SCOAP [API/XML]',
        "parser": "CernParser",
        "format": 'MARC21',
        'file_formats':["zip","xml"],
        "crawler": "CernCrawler",
   },
#   {    "type": 10,
#        "name": 'REPEC [RIS]',
#        "parser": "RISParser",
#        "format": 'RIS',
#        'file_formats':["zip","ris", "txt"],
#        "crawler": None,
#   },
#
   {    "type": 10,
        "name": 'REPEC [MULTIVAC API]',
        "parser": "MultivacParser",
        "format": 'JSON',
        'file_formats':["zip","json"],
        "crawler": "MultivacCrawler",
   },

   {    "type": 11,
        "name": 'HAL [API]',
        "parser": "HalParser",
        "format": 'JSON',
        'file_formats':["zip","json"],
        "crawler": "HalCrawler",
   },

   {    "type": 12,
        "name": 'ISIDORE [SPARQLE API /!\ BETA]',
        "parser": "IsidoreParser",
        "format": 'JSON',
        'file_formats':["zip","json"],
        "crawler": "IsidoreCrawler",
   },

]

def load_parser(resource):
    '''given a resource load the corresponding Parser
    resource(dict) > Parser(object)
    exemple with resource ISTexParser
    PARSER filename: ISTEX
    PARSER object: ISTexParser
    '''
    filename = resource["parser"].replace("Parser", '').upper()
    module = 'gargantext.util.parsers.%s' %(filename)
    module = importlib.import_module(module)
    return getattr(module, resource["parser"])


def load_crawler(resource):
    '''given a resource load the corresponding Parser()
    resource(dict) > Parser(object)
    exemple with resource ISTexCrawler
    PARSER filename: ISTEX
    PARSER object: ISTexCrawler
    '''
    filename = resource["crawler"].replace("Crawler", "").upper()
    module = 'gargantext.util.crawlers.%s' %(filename)
    module = importlib.import_module(module)
    return getattr(module, resource["crawler"])
